const axios = require("axios");
const AWS = require("aws-sdk");
const RSSParser = require("rss-parser");

// AWSクライアント初期化 (us-east-1)
const dynamoDB = new AWS.DynamoDB.DocumentClient({ region: "us-east-1" });
const ssm = new AWS.SSM({ region: "us-east-1" });
const secretsManager = new AWS.SecretsManager({ region: "us-east-1" });
const bedrock = new AWS.BedrockRuntime({ region: "us-east-1" });
const parser = new RSSParser();

// DynamoDBテーブル名
const TABLE_NAME = "RSSArticles";

// Parameter StoreからRSSサイトリストを取得
async function getRSSSites() {
  const params = { Name: "/rss_sites", WithDecryption: false };
  const response = await ssm.getParameter(params).promise();
  return response.Parameter.Value.split(",");
}

// Secrets ManagerからLINEアクセストークンを取得
async function getLineAccessToken() {
  const secret = await secretsManager.getSecretValue({
    SecretId: "<Secrets Manager ARNを入力>",
  }).promise();
  return JSON.parse(secret.SecretString).LINE_ACCESS_TOKEN;
}

// DynamoDBで新しい記事をフィルタリング
async function filterNewArticles(articles) {
  if (articles.length === 0) {
    console.log("No articles to filter.");
    return [];
  }

  const keys = articles.map(article => ({ ArticleID: article.id }));
  const params = {
    RequestItems: {
      [TABLE_NAME]: { Keys: keys },
    },
  };

  const existingArticles = await dynamoDB.batchGet(params).promise();
  const existingIds = new Set(
    existingArticles.Responses[TABLE_NAME]?.map(item => item.ArticleID) || []
  );

  return articles.filter(article => !existingIds.has(article.id));
}

// DynamoDBに記事を保存
async function saveArticles(articles) {
  if (articles.length === 0) {
    console.log("No new articles to save.");
    return;
  }

  const MAX_BATCH_SIZE = 25;
  const chunks = [];
  for (let i = 0; i < articles.length; i += MAX_BATCH_SIZE) {
    chunks.push(articles.slice(i, i + MAX_BATCH_SIZE));
  }

  for (const chunk of chunks) {
    const requests = chunk.map(article => ({
      PutRequest: {
        Item: {
          ArticleID: article.id,
          Title: article.title,
          Content: article.content,
          Timestamp: Math.floor(Date.now() / 1000),
        },
      },
    }));

    const params = { RequestItems: { [TABLE_NAME]: requests } };

    try {
      let unprocessed = await dynamoDB.batchWrite(params).promise();
      while (Object.keys(unprocessed.UnprocessedItems || {}).length > 0) {
        console.log("Retrying unprocessed items:", unprocessed.UnprocessedItems);
        unprocessed = await dynamoDB.batchWrite({
          RequestItems: unprocessed.UnprocessedItems,
        }).promise();
      }
      console.log("Articles saved to DynamoDB:", chunk);
    } catch (error) {
      console.error("Error saving articles to DynamoDB:", error);
    }
  }
}

/**
 * Amazon Bedrock (Claude 3 Haiku 20240307 Chatモデル) で要約を生成
 *   → レスポンスは "content[0].text" に要約が含まれる
 */
async function generateSummary(content) {
  const params = {
    modelId: "anthropic.claude-3-haiku-20240307-v1:0",
    body: JSON.stringify({
      // ベンダーが要求するanthropic_version
      anthropic_version: "bedrock-2023-05-31",
      max_tokens: 1000,
      messages: [
        {
          role: "user",
          content: [
            {
              type: "text",
              text: `以下の文章を要約してください:\n\n${content}`
            }
          ]
        }
      ],
    }),
    contentType: "application/json",
    accept: "application/json",
  };

  try {
    const result = await bedrock.invokeModel(params).promise();
    const responseBody = JSON.parse(result.body);

    // Claude 3 Haikuのレスポンスは "content[0].text" に要約がある
    if (responseBody.content && responseBody.content.length > 0) {
      return responseBody.content[0].text;
    } else {
      return "要約が取得できませんでした。";
    }
  } catch (error) {
    console.error("Bedrock Error:", error);
    throw new Error("Bedrockモデル呼び出しに失敗しました");
  }
}

// RSSデータの解析
async function parseRSS(data) {
  try {
    const feed = await parser.parseString(data);
    return feed.items.map(item => ({
      id: item.link,
      title: item.title,
      content: item.contentSnippet || item.content,
    }));
  } catch (error) {
    console.error("Error parsing RSS feed:", error);
    return [];
  }
}

/**
 * LINEにプッシュ通知を送信
 *   → "to" フィールド必須: 送信先ユーザー or グループID
 */
async function sendToLine(toUserId, title, summary, accessToken) {
  const message = {
    to: toUserId,
    messages: [
      {
        type: "text",
        text: `${title}\n${summary}`,
      },
    ],
  };

  try {
    await axios.post("https://api.line.me/v2/bot/message/push", message, {
      headers: {
        Authorization: `Bearer ${accessToken}`,
        "Content-Type": "application/json",
      },
    });
    console.log(`Message sent to LINE: ${title}`);
  } catch (error) {
    console.error("Failed to send LINE notification:", error);
  }
}

// メイン処理 (AWS Lambdaハンドラ)
exports.handler = async () => {
  try {
    // 1. RSSサイト一覧、LINEアクセストークン取得
    const rssSites = await getRSSSites();
    const accessToken = await getLineAccessToken();

    // 送信先ユーザーID(またはグループID)
    const toUserId = "<Uから始まるLINEのユーザーID>";

    // 2. RSS処理
    for (const site of rssSites) {
      const response = await axios.get(site);
      const articles = await parseRSS(response.data);

      // 新規記事だけ抽出 & DynamoDB保存
      const newArticles = await filterNewArticles(articles);
      await saveArticles(newArticles);

      // 3. 新規記事をBedrockで要約→LINEに送信
      for (const article of newArticles) {
        const summary = await generateSummary(article.content);
        await sendToLine(toUserId, article.title, summary, accessToken);
      }
    }

    return { statusCode: 200, body: "Success" };
  } catch (error) {
    console.error("Error:", error);
    return { statusCode: 500, body: "Error occurred" };
  }
};
